import numpy as np
import matplotlib.pyplot as plt

#==============================================================================
# #==============================================================================
# # # Time stepping schemes
# #==============================================================================
#==============================================================================

def stepEulerExplicit( rho_tk, rho_tkm1, c, ht, hx) :
    '''Explicit Euler scheme'''

    # TODO

    return  rho_tk

def stepEulerImplicit( rho_tk, rho_tkm1, c, ht, hx) :
    '''Implicit Euler scheme'''

    # TODO

    return rho_tk


def stepLeapFrog( rho_tk,  rho_tkm1, c, ht, hx) :
    '''Leap-Frog scheme'''

    # TODO

    return rho_tk


def stepCrankNicolson( rho_tk,  rho_tkm1, c, ht, hx):
    '''Crank Nicolson scheme'''
    
    # TODO
    
    return rho_tk


def stepLaxWendroff( rho_tk, rho_tkm1, c, ht, hx) :
    '''Lax Wendroff scheme'''
    
    # TODO
    
    return rho_tk

#==============================================================================
# #==============================================================================
# # #compute reference solution and initial conditions
# #==============================================================================
#==============================================================================

def computeRefSolution( ini_cond_name, t_interval = [0,1], x_interval = [-5, 5], ht = 0.01, hx = 0.01):

    ts = np.arange( t_interval[0], t_interval[1], ht)
    xk1 = np.arange( x_interval[0], x_interval[1], hx)
    sol_ref = np.zeros(( ts.size, xk1.size))
    
 
    # TODO
    
    return sol_ref

# simulation loop


def simulate( integrator, ini_cond, c, t_interval=[0, 1], x_interval = [-5,5], ht = 0.01, hx = 0.01) :
    '''Run simulation'''

    ts = np.arange( t_interval[0], t_interval[1], ht)
    xk1 = np.arange( x_interval[0], x_interval[1], hx)
    N = ini_cond.size

    sol = np.zeros(( ts.size, xk1.size))
   
    rho_tk = ini_cond
    rho_tk_m1 = stepEulerExplicit(rho_tk, np.array([0]) ,-c, ht, hx)

    for tid, t in enumerate(ts) :
        
        cur = np.copy(rho_tk)
        rho_tk = integrator( rho_tk, rho_tk_m1, c, ht, hx)
        rho_tk_m1 = np.copy(cur)

        sol[tid] = rho_tk.reshape((1, N))
       
    return sol, ts

#==============================================================================
#==============================================================================
# # # main
#==============================================================================
#==============================================================================

#setup

c = -1.0 #speed
ini_cond_name = 'gaussian'
t_interval = [0, 5]
x_interval = [-10, 10]
t0, te = t_interval[0], t_interval[1]
x0, xe = x_interval[0], x_interval[1]
ht = 0.005
hx = 0.01

sol_ref = computeRefSolution( ini_cond_name, t_interval, x_interval, ht, hx )
ini_cond = sol_ref[0]

xx = np.arange( x0, xe, hx)

#plotting reference solution

fig = plt.figure()
plt.plot( xx, sol_ref[0])
plt.plot( xx, sol_ref[ int (sol_ref.shape[0]/5 )])
plt.plot( xx, sol_ref[ int (2*sol_ref.shape[0]/5 )])
plt.plot( xx, sol_ref[ int (3*sol_ref.shape[0]/5 )])
plt.plot( xx, sol_ref[ int (4*sol_ref.shape[0]/5 )])
plt.plot( xx, sol_ref[ int (sol_ref.shape[0]-1)])
plt.legend( ['t=0', 't=1', 't=2', 't=3', 't=4', 't=5'])
plt.title( 'Reference solution')
plt.show()
fig.savefig( './transport_reference.pdf')

# basic experiments
print( "Starting basic experiments.")

integrators = [stepEulerExplicit, stepEulerImplicit, stepLeapFrog, stepCrankNicolson, stepLaxWendroff]
integrator_labels = [ 'Explicit Euler', 'Implicit Euler', 'Leap Frog', 'Crank Nicolson', 'Lax Wendroff']
sols = []


for integrator in integrators :
    sol, tlist = simulate( integrator, ini_cond, c, t_interval, x_interval, ht, hx)
    sols.append( sol)

#plotting solutions and error measurements
scheme = 0
for sol in sols:

    fig, axs = plt.subplots(1, 2, figsize=(15, 5), sharex=True)

    axs[0].plot( xx, sol[0])
    axs[0].plot( xx, sol[int(sol.shape[0]/4)])
    axs[0].plot( xx, sol[int(2*sol.shape[0]/4)])
    axs[0].plot( xx, sol[int(3*sol.shape[0]/4)])
    axs[0].plot( xx, sol[int(sol.shape[0]-1)])
    axs[0].legend( ['t=0', 't=0.25', 't=0.5', 't=0.75', 't=1.'])
    axs[0].set_title(integrator_labels[scheme])
    axs[0].grid(True)

    axs[1].plot(xx,np.abs(sol[0] - sol_ref[0]) )
    axs[1].plot( xx, np.abs(sol[int(sol.shape[0]/4)] - sol_ref[int(sol.shape[0]/4)]))
    axs[1].plot( xx, np.abs(sol[int(2*sol.shape[0]/4)] - sol_ref[int(2*sol.shape[0]/4)]))
    axs[1].plot( xx, np.abs(sol[int(3*sol.shape[0]/4)] - sol_ref[int(3*sol.shape[0]/4)]))
    axs[1].plot( xx, np.abs(sol[int(sol.shape[0]-1)] - sol_ref[int(sol.shape[0]-1)]))
    axs[1].legend( ['t=0', 't=0.25', 't=0.5', 't=0.75', 't=1.'])
    axs[1].set_title(' error ( h_t = ' +str(ht) + ', h_x = '+str(hx) + ' )')
    axs[1].grid(True)

    plt.subplots_adjust(wspace=0.3)
    fig.savefig( './transport_'+ini_cond_name +'_'+ integrator_labels[scheme]+'.png')

    plt.show()
    plt.close(fig)
    scheme+=1
